@@ -2,11 +2,13 @@ from .api.sql_connection import SQLConnection
 from .read.query_constructor import QueryConstructor
 from .write.create_tables import IngestDataBase
 from .write.make_universes import compustat_us_universe, crsp_us_universe
+from .read.db_functions import table_info
 
 __all__ = [
     'SQLConnection',
     'QueryConstructor',
     'IngestDataBase',
     'compustat_us_universe',
-    'crsp_us_universe'
+    'crsp_us_universe',
+    'table_info'
 ]
