@@ -1,10 +1,11 @@
 import glob
 import os.path
+
 import pandas as pd
 
 from typing import Union
 
-from toolbox.db.settings import CACHE_DIRECTORY
+from toolbox.db.settings import ADD_ALL_LINKS_TO_PERMNO, CACHE_DIRECTORY
 from toolbox.db.api.sql_connection import SQLConnection
 
 # this allows compatibility with python 3.6
@@ -29,7 +30,7 @@ class ETFUniverse:
         :param con: A connection to the database, if not passed then will open a new write connection.
             if the there is universe creation going on then must pass a write connection
         """
-        self._con = con if con else SQLConnection(read_only=False)
+        self._con = con if con else SQLConnection()
 
     def get_universe_df(self, ticker: str = None, crsp_portno: int = None, start_date: str = '2000',
                         end_date: str = '2023') -> Union[pd.DataFrame, str]:
@@ -99,6 +100,7 @@ class ETFUniverse:
         will cache etf in temp directory of the computer
         :return: pd.Dataframe index: int_range; columns: date, permno;
         """
+        print('Caching ETF Holdings')
 
         sql_for_holdings = f"""
                 SELECT DISTINCT date, permno 
@@ -111,7 +113,7 @@ class ETFUniverse:
         df_of_holdings = raw_etf_holdings.set_index('date').groupby('date')['permno'].apply(
             lambda grp: list(grp.value_counts().index))
 
-        end_date = pd.to_datetime('now').date().strftime('%Y-%m-%d')
+        end_date = pd.Timestamp.now().date().strftime('%Y-%m-%d')
         start_date = df_of_holdings.index.min()
 
         full_cal = pd.date_range(start=start_date, end=end_date, freq='D').tz_localize(None)
@@ -120,15 +122,28 @@ class ETFUniverse:
             'NYSE').valid_days(start_date=start_date, end_date=end_date).tz_localize(
             None)
 
-        universe = df_of_holdings.reindex(full_cal.tolist()).ffill().reindex(trading_cal.tolist())
+        universe = df_of_holdings.reindex(full_cal.tolist()).ffill().reindex(trading_cal.tolist()).reset_index()
 
         relational_format = [(row[0], permno) for row in universe.values for permno in row[1]]
         uni_df = pd.DataFrame(relational_format, columns=['date', 'permno'])
+        uni_df = self._link_to_ids(uni_df)
 
         self._cache_helper(uni_df=uni_df, crsp_portno=crsp_portno)
 
         return uni_df
 
+    def _link_to_ids(self, uni_df: pd.DataFrame) -> pd.DataFrame:
+        """
+        join cstat and ibes links to current universe df
+        """
+        columns = ', '.join(['date', 'uni.permno', 'gvkey', 'liid as iid', 'ticker', 'cusip',
+                             "CASE WHEN gvkey NOT NULL THEN CONCAT(gvkey, '_', liid) ELSE NULL END as id"])
+        from_start = " uni_df as uni "
+
+        sql_code = ADD_ALL_LINKS_TO_PERMNO.replace('--columns', columns).replace('--from', from_start)
+
+        return self._con.con.execute(sql_code).fetchdf()
+
     def _get_crsp_portno(self, ticker, crsp_portno) -> int:
         """
         if ticker is not none then will map the ticker to a crsp_portno
@@ -147,8 +162,15 @@ class ETFUniverse:
             raise ValueError(f"Ticker '{ticker}' is not valid cant map to crsp_portno")
 
         if len(mapped_id) > 1:
+            # getting metadata of the portno's that mtched
+            mapped_funds = self._con.execute(f"""SELECT DISTINCT crsp_portno, fund_name, m_fund, et_flag
+                                            FROM crsp.fund_summary 
+                                            WHERE ticker = '{ticker}' AND
+                                                crsp_portno IS NOT NULL""").fetchdf()
+
             raise ValueError(f"Ticker '{ticker}' mapped to {len(mapped_id)} crsp_portno's {mapped_id}. "
-                             f"Please specify the crsp_portno to build this etf's history")
+                             f"Please specify the crsp_portno to build this etf's history\n" +
+                              mapped_funds.to_string(index=False))
 
         return int(mapped_id[0][0])
 
@@ -199,3 +221,7 @@ def clear_cache():
     for f in files:
         os.remove(f)
     print('Cleared Cache')
+
+
+if __name__ == '__main__':
+    print(ETFUniverse().get_universe_df(crsp_portno=1025812))
